import decimal
import logging
import typing
import uuid

import inflection
import sqlalchemy as sa
from hitfactorpy.enums import Classification, Division, MatchLevel, PowerFactor, Scoring
from hitfactorpy.utils import calculate_hit_factor
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.ext.hybrid import hybrid_property
from sqlalchemy.orm import declarative_base, declarative_mixin, declared_attr, relationship, validates  # type: ignore
from sqlalchemy.orm.attributes import Mapped  # type: ignore
from sqlalchemy.orm.exc import DetachedInstanceError
from sqlalchemy_continuum import make_versioned

from .expression import greatest

_logger = logging.getLogger(__name__)

make_versioned(user_cls=None)

"""
https://docs.sqlalchemy.org/en/20/core/type_basics.html#sqlalchemy.types.Enum.params.name
"""
ENUM_PREFIX = "HITFACTORPY_ENUM_"


def _validate_positive_int(value: int, name: str):
    if not value >= 0:
        raise ValueError(f'value for "{name}" must be >=0, but received: {value}')
    return value


def _validate_positive_decimal(value: int | float | decimal.Decimal, name: str):
    if not value >= 0.0:
        raise ValueError(f'value for "{name}" must be >=0.0, but received: {value}')
    return value


class _BaseCls:
    """https://docs.sqlalchemy.org/en/13/orm/extensions/declarative/mixins.html#augmenting-the-base"""

    @declared_attr
    def __tablename__(cls):
        return inflection.underscore(cls.__name__)


"""Base ORM Model"""
SABaseModel = declarative_base(cls=_BaseCls)


@declarative_mixin
class MixinVersioned:
    """Mixin requirements for sqlalchemy-continuum"""

    @declared_attr
    def __versioned__(cls):
        return {}


@declarative_mixin
class MixinIds:
    """Mixin columns for a database-internal id as the primary key, and a public uuid"""

    internal_id = sa.Column(
        sa.Integer,
        primary_key=True,
        autoincrement=True,
        comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
    )
    id = sa.Column(
        UUID(as_uuid=True),
        nullable=False,
        unique=True,
        index=True,
        server_default=sa.text("gen_random_uuid()"),
        comment="ID exposed in public APIs",
    )


class BaseModel(SABaseModel, MixinIds):  # type: ignore
    __abstract__ = True

    def __repr__(self) -> str:
        return self._repr(id=self.id)  # type: ignore

    def _repr(self, **fields: typing.Dict[str, typing.Any]) -> str:
        """
        Helper for __repr__: https://stackoverflow.com/a/55749579
        """
        field_strings = []
        at_least_one_attached_attribute = False
        for key, field in fields.items():
            try:
                field_strings.append(f"{key}={field!r}")
            except DetachedInstanceError:
                field_strings.append(f"{key}=DetachedInstanceError")
            else:
                at_least_one_attached_attribute = True
        if at_least_one_attached_attribute:
            return f"<{self.__class__.__name__}({','.join(field_strings)})>"
        return f"<{self.__class__.__name__} {id(self)}>"


class VersionedModel(BaseModel):  # , MixinVersioned):
    __abstract__ = True


SAEnumMatchLevel = sa.Enum(MatchLevel, create_constraint=True, name=f"{ENUM_PREFIX}MatchReportMatchLevel")
SAEnumDivision = sa.Enum(Division, create_constraint=True, name=f"{ENUM_PREFIX}MatchReportCompetitorDivision")
SAEnumClassification = sa.Enum(
    Classification, create_constraint=True, name=f"{ENUM_PREFIX}MatchReportCompetitorClassification"
)
SAEnumPowerFactor = sa.Enum(PowerFactor, create_constraint=True, name=f"{ENUM_PREFIX}MatchReportCompetitorPowerFactor")
SAEnumScoring = sa.Enum(Scoring, create_constraint=True, name=f"{ENUM_PREFIX}MatchReportStageScoring")


class MatchReport(VersionedModel):
    """Parsed contents of a match report"""

    # Columns
    name = sa.Column(sa.Unicode(255), nullable=False)
    date = sa.Column(sa.Date)
    match_level = sa.Column(SAEnumMatchLevel)
    report_hash = sa.Column(
        UUID(as_uuid=True),
        nullable=True,
        unique=True,
        comment="This represents an md5 hexdigest, stored as a UUID. Used to identify potential duplicate report imports.",
    )

    # Relationships
    competitors: Mapped["MatchReportCompetitor"] = relationship(
        "MatchReportCompetitor",
        back_populates="match",
        cascade="all, delete",
        passive_deletes=True,
        primaryjoin="MatchReport.internal_id==MatchReportCompetitor.match_internal_id",
    )
    stages: Mapped["MatchReportStage"] = relationship(
        "MatchReportStage",
        back_populates="match",
        cascade="all, delete",
        passive_deletes=True,
        primaryjoin="MatchReport.internal_id==MatchReportStage.match_internal_id",
    )
    stage_scores: Mapped["MatchReportStageScore"] = relationship(
        "MatchReportStageScore",
        back_populates="match",
        cascade="all, delete",
        passive_deletes=True,
        primaryjoin="MatchReport.internal_id==MatchReportStageScore.match_internal_id",
    )

    # Validators
    @validates("report_hash")
    def validate_report_hash(self, key, value):
        if value and isinstance(value, str) and len(value) == 32:
            return uuid.UUID(value)
        _logger.warning("report_hash('%s') failed validation: '%s'", key, value)
        return None


class MatchReportCompetitor(VersionedModel):
    # Columns
    match_internal_id = sa.Column(
        sa.Integer, sa.ForeignKey(MatchReport.internal_id, ondelete="CASCADE"), nullable=False
    )
    member_number = sa.Column(sa.Unicode(64))
    first_name = sa.Column(sa.Unicode(64))
    last_name = sa.Column(sa.Unicode(64))
    division = sa.Column(SAEnumDivision)
    classification = sa.Column(SAEnumClassification)
    power_factor = sa.Column(SAEnumPowerFactor)
    dq = sa.Column(sa.Boolean, nullable=False, default=False)
    reentry = sa.Column(sa.Boolean, nullable=False, default=False)

    # Relationships
    match: Mapped[MatchReport] = relationship(MatchReport, uselist=False, back_populates="competitors")
    stage_scores: Mapped["MatchReportStageScore"] = relationship("MatchReportStageScore", back_populates="competitor")


class MatchReportStage(VersionedModel):
    # Columns
    match_internal_id = sa.Column(
        sa.Integer, sa.ForeignKey(MatchReport.internal_id, ondelete="CASCADE"), nullable=False
    )
    name = sa.Column(sa.Unicode(255))
    min_rounds = sa.Column(sa.Integer, nullable=False)
    max_points = sa.Column(sa.Integer, nullable=False)
    classifier = sa.Column(sa.Boolean, nullable=False, default=False)
    classifier_number = sa.Column(sa.Unicode(64))
    stage_number = sa.Column(sa.Integer)
    scoring_type = sa.Column(
        SAEnumScoring,
        nullable=False,
        default=Scoring.COMSTOCK,
    )

    # relationships
    match: Mapped[MatchReport] = relationship(MatchReport, uselist=False, back_populates="stages")
    stage_scores: Mapped["MatchReportStageScore"] = relationship(
        "MatchReportStageScore",
        back_populates="stage",
        cascade="all, delete",
        passive_deletes=True,
        primaryjoin="MatchReportStage.internal_id==MatchReportStageScore.stage_internal_id",
    )

    # Validators
    @validates("min_rounds")
    def validate_min_rounds(self, key, value):
        return _validate_positive_int(value, key)

    @validates("max_points")
    def validate_max_points(self, key, value):
        return _validate_positive_int(value, key)

    # Constraints
    __table_args__ = (
        (sa.UniqueConstraint("match_internal_id", "stage_number", name="stage_score_stage_number_unique_per_match")),
    )


class MatchReportStageScore(VersionedModel):
    # Columns
    match_internal_id = sa.Column(
        sa.Integer, sa.ForeignKey(MatchReport.internal_id, ondelete="CASCADE"), nullable=False
    )
    competitor_internal_id = sa.Column(
        sa.Integer, sa.ForeignKey(MatchReportCompetitor.internal_id, ondelete="CASCADE"), nullable=False
    )
    stage_internal_id = sa.Column(
        sa.Integer, sa.ForeignKey(MatchReportStage.internal_id, ondelete="CASCADE"), nullable=False
    )
    dq = sa.Column(sa.Boolean, nullable=False, default=False)
    dnf = sa.Column(sa.Boolean, nullable=False, default=False)
    a = sa.Column(sa.Integer, nullable=False, default=0)
    b = sa.Column(sa.Integer, nullable=False, default=0)
    c = sa.Column(sa.Integer, nullable=False, default=0)
    d = sa.Column(sa.Integer, nullable=False, default=0)
    m = sa.Column(sa.Integer, nullable=False, default=0)
    npm = sa.Column(sa.Integer, nullable=False, default=0)
    ns = sa.Column(sa.Integer, nullable=False, default=0)
    procedural = sa.Column(sa.Integer, nullable=False, default=0)
    late_shot = sa.Column(sa.Integer, nullable=False, default=0)
    extra_shot = sa.Column(sa.Integer, nullable=False, default=0)
    extra_hit = sa.Column(sa.Integer, nullable=False, default=0)
    other_penalty = sa.Column(sa.Integer, nullable=False, default=0)
    t1 = sa.Column(
        sa.Numeric(precision=8, scale=2, asdecimal=True, decimal_return_scale=2), nullable=False, default=0.0
    )
    t2 = sa.Column(
        sa.Numeric(precision=8, scale=2, asdecimal=True, decimal_return_scale=2), nullable=False, default=0.0
    )
    t3 = sa.Column(
        sa.Numeric(precision=8, scale=2, asdecimal=True, decimal_return_scale=2), nullable=False, default=0.0
    )
    t4 = sa.Column(
        sa.Numeric(precision=8, scale=2, asdecimal=True, decimal_return_scale=2), nullable=False, default=0.0
    )
    t5 = sa.Column(
        sa.Numeric(precision=8, scale=2, asdecimal=True, decimal_return_scale=2), nullable=False, default=0.0
    )
    time = sa.Column(
        sa.Numeric(precision=8, scale=2, asdecimal=True, decimal_return_scale=2), nullable=False, default=0.0
    )
    raw_points = sa.Column(sa.Numeric(precision=8, scale=2, asdecimal=True, decimal_return_scale=2))
    penalty_points = sa.Column(sa.Numeric(precision=8, scale=2, asdecimal=True, decimal_return_scale=2))
    total_points = sa.Column(sa.Numeric(precision=8, scale=2, asdecimal=True, decimal_return_scale=2))
    hit_factor = sa.Column(sa.Numeric(precision=8, scale=4, asdecimal=True, decimal_return_scale=4))
    stage_points = sa.Column(sa.Numeric(precision=8, scale=4, asdecimal=True, decimal_return_scale=4))
    stage_place = sa.Column(sa.Integer)
    stage_power_factor = sa.Column(SAEnumPowerFactor)

    # Relationships
    match: Mapped[MatchReport] = relationship(MatchReport, uselist=False, back_populates="stage_scores")
    competitor: Mapped[MatchReportCompetitor] = relationship(
        MatchReportCompetitor, uselist=False, back_populates="stage_scores"
    )
    stage: Mapped[MatchReportStage] = relationship(MatchReportStage, uselist=False, back_populates="stage_scores")

    @hybrid_property
    def calculated_power_factor(self):
        """Power factor is reported on the competitor, by default, but some stages can override the power factor via `stage_power_factor`."""
        return (
            self.stage_power_factor
            if self.stage_power_factor and self.stage_power_factor != PowerFactor.UNKNOWN
            else self.competitor.power_factor
        )

    @calculated_power_factor.expression  # type: ignore
    def calculated_power_factor(cls):
        return sa.case(
            (cls.stage_power_factor == PowerFactor.MAJOR, PowerFactor.MAJOR),
            (cls.stage_power_factor == PowerFactor.MINOR, PowerFactor.MINOR),
            (MatchReportCompetitor.power_factor == PowerFactor.MAJOR, PowerFactor.MAJOR),
            else_=PowerFactor.MINOR,
        )

    @hybrid_property
    def calculated_hit_factor(self):
        return calculate_hit_factor(
            scoring_type=self.stage.scoring_type,
            power_factor=self.calculated_power_factor,
            dq=self.dq or self.competitor.dq,
            dnf=self.dnf,
            a=self.a,
            c=self.c,
            d=self.d,
            m=self.m,
            ns=self.ns,
            procedural=self.procedural,
            other_penalty=self.other_penalty,
            late_shot=self.late_shot,
            extra_shot=self.extra_shot,
            extra_hit=self.extra_hit,
            time=self.time,
            hit_factor=self.hit_factor,
        )

    @calculated_hit_factor.expression  # type: ignore
    def calculated_hit_factor(cls):
        return (
            sa.select(
                sa.case(
                    (
                        MatchReportStage.scoring_type == Scoring.CHRONO,
                        sa.cast(cls.hit_factor, sa.Numeric(precision=8, scale=4, decimal_return_scale=4)),
                    ),
                    else_=greatest(
                        sa.case(
                            (
                                sa.or_(cls.dq, cls.dnf, MatchReportCompetitor.dq),
                                sa.cast(0.0, sa.Numeric(precision=8, scale=4, decimal_return_scale=4)),
                            ),
                            else_=sa.cast(
                                (
                                    (
                                        (cls.a * 5)
                                        + (
                                            cls.c
                                            * sa.case(
                                                (cls.calculated_power_factor == PowerFactor.MAJOR, 4),
                                                else_=3,
                                            )
                                        )
                                        + (
                                            cls.d
                                            * sa.case(
                                                (cls.calculated_power_factor == PowerFactor.MAJOR, 2),
                                                else_=1,
                                            )
                                        )
                                        + (cls.m * -10)
                                        + (cls.ns * -10)
                                        + (cls.procedural * -10)
                                        + (cls.other_penalty * -1)
                                        + (cls.late_shot * -5)
                                        + (cls.extra_shot * -10)
                                        + (cls.extra_hit * -10)
                                    )
                                    / sa.case((cls.time > 0, cls.time), else_=1)
                                ),
                                sa.Numeric(precision=8, scale=4, decimal_return_scale=4),
                            ),
                        ),
                        sa.cast(0.0, sa.Numeric(precision=8, scale=4, decimal_return_scale=4)),
                    ),
                )
            )
            .where(MatchReportCompetitor.id == cls.competitor_id)
            .where(MatchReportStage.id == cls.stage_id)
            .label("calculated_hit_factor")
        )

    # Validators
    @validates("a")
    def validate_a(self, key, value):
        return _validate_positive_int(value, key)

    @validates("b")
    def validate_b(self, key, value):
        return _validate_positive_int(value, key)

    @validates("c")
    def validate_c(self, key, value):
        return _validate_positive_int(value, key)

    @validates("d")
    def validate_d(self, key, value):
        return _validate_positive_int(value, key)

    @validates("m")
    def validate_m(self, key, value):
        return _validate_positive_int(value, key)

    @validates("ns")
    def validate_ns(self, key, value):
        return _validate_positive_int(value, key)

    @validates("npm")
    def validate_npm(self, key, value):
        return _validate_positive_int(value, key)

    @validates("hit_factor")
    def validate_hit_factor(self, key, value):
        return _validate_positive_decimal(value, key)

    # __table_args__ = sa.CheckConstraint("col2 > col3 + 5", name="check1")


sa.orm.configure_mappers()  # Must be called immediately after the last model definition



from pathlib import Path

from alembic.config import Config as AlembicConfig

from .defaults import HITFACTORPY_DB_ALEMBIC_DIR
from .session import get_sqlalchemy_url


def HitfactorpyAlembicConfig(
    script_location: str | Path = HITFACTORPY_DB_ALEMBIC_DIR, sqlalchemy_url: str = get_sqlalchemy_url()
) -> AlembicConfig:
    alembic_cfg = AlembicConfig()
    alembic_cfg.set_main_option("script_location", str(script_location))
    alembic_cfg.set_main_option("sqlalchemy.url", sqlalchemy_url)
    return alembic_cfg



import typer

from .import_ import cli as import_cli
from .migrate import cli as migrate_cli
from .model import cli as model_cli

CLI_NAME = "hitfactorpy_sqlalchemy"


cli = typer.Typer()


def version_callback(value: bool):
    from .. import __version__ as pkg_version

    if value:
        typer.echo(pkg_version)
        raise typer.Exit()


@cli.callback()
def main(
    version: bool = typer.Option(
        None, "--version", callback=version_callback, is_eager=True, help="Show program version and exit"
    ),
):
    pass


cli.add_typer(import_cli, name="import", help="import data")
cli.add_typer(migrate_cli, name="migrate", help="manage db migrations")
cli.add_typer(model_cli, name="model", help="manage models directly")


if __name__ == "__main__":
    cli()



from dataclasses import dataclass
from pathlib import Path

import typer
from alembic import command as alembic_command
from alembic.config import Config as AlembicConfig
from alembic.util.exc import AutogenerateDiffsDetected, CommandError
from sqlalchemy.engine.url import URL

from .. import defaults as _defaults
from .. import env as _env
from ..alembic_utils import HitfactorpyAlembicConfig

cli = typer.Typer()


@dataclass(frozen=True)
class CtxDbConnection:
    scheme: str
    username: str
    password: str
    host: str
    port: int
    database: str


@dataclass(frozen=True)
class CtxObj:
    alembic_dir: Path
    db_connection: CtxDbConnection
    sqlalchemy_url: str
    alembic_cfg: AlembicConfig


@cli.callback()
def inject_shared_options(
    ctx: typer.Context,
    alembic_dir: Path = typer.Option(_defaults.HITFACTORPY_DB_ALEMBIC_DIR, envvar=_env.HITFACTORPY_DB_ALEMBIC_DIR),
    username: str = typer.Option(
        _defaults.HITFACTORPY_DB_USERNAME, help="Database username", envvar=_env.HITFACTORPY_DB_USERNAME
    ),
    password: str = typer.Option(
        _defaults.HITFACTORPY_DB_PASSWORD, help="Database password", envvar=_env.HITFACTORPY_DB_USERNAME
    ),
    host: str = typer.Option(_defaults.HITFACTORPY_DB_HOST, help="Database host", envvar=_env.HITFACTORPY_DB_HOST),
    port: int = typer.Option(_defaults.HITFACTORPY_DB_PORT, help="Database port", envvar=_env.HITFACTORPY_DB_PORT),
    database: str = typer.Option(
        _defaults.HITFACTORPY_DB_DATABASE_NAME, help="Database name", envvar=_env.HITFACTORPY_DB_DATABASE_NAME
    ),
):
    db_conn = CtxDbConnection(
        scheme="postgresql",
        username=username,
        password=password,
        host=host,
        port=port,
        database=database,
    )
    sqlalchemy_url = str(
        URL.create(  # type: ignore
            "postgresql+asyncpg",  # must use an async driver with this alembic config
            username=db_conn.username,
            password=db_conn.password,
            host=db_conn.host,
            port=db_conn.port,
            database=db_conn.database,
        )
    )
    alembic_cfg = HitfactorpyAlembicConfig(script_location=alembic_dir, sqlalchemy_url=sqlalchemy_url)
    ctx.obj = CtxObj(
        alembic_dir=alembic_dir,
        db_connection=db_conn,
        alembic_cfg=alembic_cfg,
        sqlalchemy_url=sqlalchemy_url,
    )


@cli.command()
def check(ctx: typer.Context):
    """alembic check"""
    config: CtxObj = ctx.obj

    try:
        alembic_command.check(config.alembic_cfg)
    except (AutogenerateDiffsDetected, CommandError) as e:
        typer.echo(e)
        raise typer.Exit(1)
    typer.echo("OK: DB and code are in sync")


@cli.command()
def upgrade(ctx: typer.Context, revision: str = "head"):
    """alembic upgrade <revision>"""
    config: CtxObj = ctx.obj

    try:
        alembic_command.upgrade(config.alembic_cfg, revision)
    except (AutogenerateDiffsDetected, CommandError) as e:
        typer.echo(e)
        raise typer.Exit(1)
    typer.echo("OK: migration upgrade applied")


@cli.command()
def downgrade(ctx: typer.Context, revision: str = "-1"):
    """alembic downgrade <revision>"""
    config: CtxObj = ctx.obj
    typer.confirm("Are you sure you want to downgrade?", abort=True)

    try:
        alembic_command.downgrade(config.alembic_cfg, revision)
    except (AutogenerateDiffsDetected, CommandError) as e:
        typer.echo(e)
        raise typer.Exit(1)

    typer.echo("OK: migration downgrade applied")


if __name__ == "__main__":
    cli()



from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql import case
from sqlalchemy.sql.expression import FunctionElement  # type: ignore
from sqlalchemy.types import Numeric


class greatest(FunctionElement):
    """See: https://docs.sqlalchemy.org/en/20/core/compiler.html#greatest-function"""

    type = Numeric()
    name = "greatest"
    inherit_cache = True


@compiles(greatest)
def default_greatest(element, compiler, **kw):
    return compiler.visit_function(element)


@compiles(greatest, "sqlite")
@compiles(greatest, "mssql")
@compiles(greatest, "oracle")
def case_greatest(element, compiler, **kw):
    arg1, arg2 = list(element.clauses)
    return compiler.process(case([(arg1 > arg2, arg1)], else_=arg2), **kw)



"""init

Revision ID: 6b0dd11d4b71
Revises: 
Create Date: 2023-02-14 13:03:25.553543

"""  # noqa: W291
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "6b0dd11d4b71"
down_revision = None
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    from hitfactorpy_sqlalchemy.orm.models import (
        SAEnumMatchLevel,
        SAEnumClassification,
        SAEnumDivision,
        SAEnumPowerFactor,
        SAEnumScoring,
    )

    op.create_table(
        "match_report",
        sa.Column(
            "internal_id",
            sa.Integer(),
            autoincrement=True,
            nullable=False,
            comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        ),
        sa.Column(
            "id",
            postgresql.UUID(as_uuid=True),
            server_default=sa.text("gen_random_uuid()"),
            nullable=False,
            comment="ID exposed in public APIs",
        ),
        sa.Column("name", sa.Unicode(length=255), nullable=False),
        sa.Column("date", sa.Date(), nullable=True),
        sa.Column("match_level", SAEnumMatchLevel, nullable=True),
        sa.Column(
            "report_hash",
            postgresql.UUID(as_uuid=True),
            nullable=True,
            comment="This represents an md5 hexdigest, stored as a UUID. Used to identify potential duplicate report imports.",
        ),
        sa.PrimaryKeyConstraint("internal_id"),
        sa.UniqueConstraint("report_hash"),
    )
    op.create_index(op.f("ix_match_report_id"), "match_report", ["id"], unique=True)
    op.create_table(
        "match_report_competitor",
        sa.Column(
            "internal_id",
            sa.Integer(),
            autoincrement=True,
            nullable=False,
            comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        ),
        sa.Column(
            "id",
            postgresql.UUID(as_uuid=True),
            server_default=sa.text("gen_random_uuid()"),
            nullable=False,
            comment="ID exposed in public APIs",
        ),
        sa.Column("match_internal_id", sa.Integer(), nullable=False),
        sa.Column("member_number", sa.Unicode(length=64), nullable=True),
        sa.Column("first_name", sa.Unicode(length=64), nullable=True),
        sa.Column("last_name", sa.Unicode(length=64), nullable=True),
        sa.Column("division", SAEnumDivision, nullable=True),
        sa.Column("classification", SAEnumClassification, nullable=True),
        sa.Column("power_factor", SAEnumPowerFactor, nullable=True),
        sa.Column("dq", sa.Boolean(), nullable=False),
        sa.Column("reentry", sa.Boolean(), nullable=False),
        sa.ForeignKeyConstraint(["match_internal_id"], ["match_report.internal_id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("internal_id"),
    )
    op.create_index(op.f("ix_match_report_competitor_id"), "match_report_competitor", ["id"], unique=True)
    op.create_table(
        "match_report_stage",
        sa.Column(
            "internal_id",
            sa.Integer(),
            autoincrement=True,
            nullable=False,
            comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        ),
        sa.Column(
            "id",
            postgresql.UUID(as_uuid=True),
            server_default=sa.text("gen_random_uuid()"),
            nullable=False,
            comment="ID exposed in public APIs",
        ),
        sa.Column("match_internal_id", sa.Integer(), nullable=False),
        sa.Column("name", sa.Unicode(length=255), nullable=True),
        sa.Column("min_rounds", sa.Integer(), nullable=False),
        sa.Column("max_points", sa.Integer(), nullable=False),
        sa.Column("classifier", sa.Boolean(), nullable=False),
        sa.Column("classifier_number", sa.Unicode(length=64), nullable=True),
        sa.Column("stage_number", sa.Integer(), nullable=True),
        sa.Column("scoring_type", SAEnumScoring, nullable=False),
        sa.ForeignKeyConstraint(["match_internal_id"], ["match_report.internal_id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("internal_id"),
        sa.UniqueConstraint("match_internal_id", "stage_number", name="stage_score_stage_number_unique_per_match"),
    )
    op.create_index(op.f("ix_match_report_stage_id"), "match_report_stage", ["id"], unique=True)
    op.create_table(
        "match_report_stage_score",
        sa.Column(
            "internal_id",
            sa.Integer(),
            autoincrement=True,
            nullable=False,
            comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        ),
        sa.Column(
            "id",
            postgresql.UUID(as_uuid=True),
            server_default=sa.text("gen_random_uuid()"),
            nullable=False,
            comment="ID exposed in public APIs",
        ),
        sa.Column("match_internal_id", sa.Integer(), nullable=False),
        sa.Column("competitor_internal_id", sa.Integer(), nullable=False),
        sa.Column("stage_internal_id", sa.Integer(), nullable=False),
        sa.Column("dq", sa.Boolean(), nullable=False),
        sa.Column("dnf", sa.Boolean(), nullable=False),
        sa.Column("a", sa.Integer(), nullable=False),
        sa.Column("b", sa.Integer(), nullable=False),
        sa.Column("c", sa.Integer(), nullable=False),
        sa.Column("d", sa.Integer(), nullable=False),
        sa.Column("m", sa.Integer(), nullable=False),
        sa.Column("npm", sa.Integer(), nullable=False),
        sa.Column("ns", sa.Integer(), nullable=False),
        sa.Column("procedural", sa.Integer(), nullable=False),
        sa.Column("late_shot", sa.Integer(), nullable=False),
        sa.Column("extra_shot", sa.Integer(), nullable=False),
        sa.Column("extra_hit", sa.Integer(), nullable=False),
        sa.Column("other_penalty", sa.Integer(), nullable=False),
        sa.Column("t1", sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=False),
        sa.Column("t2", sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=False),
        sa.Column("t3", sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=False),
        sa.Column("t4", sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=False),
        sa.Column("t5", sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=False),
        sa.Column("time", sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=False),
        sa.Column("raw_points", sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=True),
        sa.Column("penalty_points", sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=True),
        sa.Column("total_points", sa.Numeric(precision=8, scale=2, decimal_return_scale=2), nullable=True),
        sa.Column("hit_factor", sa.Numeric(precision=8, scale=4, decimal_return_scale=4), nullable=True),
        sa.Column("stage_points", sa.Numeric(precision=8, scale=4, decimal_return_scale=4), nullable=True),
        sa.Column("stage_place", sa.Integer(), nullable=True),
        sa.Column("stage_power_factor", SAEnumPowerFactor, nullable=True),
        sa.ForeignKeyConstraint(
            ["competitor_internal_id"], ["match_report_competitor.internal_id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["match_internal_id"], ["match_report.internal_id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["stage_internal_id"], ["match_report_stage.internal_id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("internal_id"),
    )
    op.create_index(op.f("ix_match_report_stage_score_id"), "match_report_stage_score", ["id"], unique=True)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    from hitfactorpy_sqlalchemy.orm.models import (
        SAEnumMatchLevel,
        SAEnumClassification,
        SAEnumDivision,
        SAEnumPowerFactor,
        SAEnumScoring,
    )

    op.drop_index(op.f("ix_match_report_stage_score_id"), table_name="match_report_stage_score")
    op.drop_table("match_report_stage_score")
    op.drop_index(op.f("ix_match_report_stage_id"), table_name="match_report_stage")
    op.drop_table("match_report_stage")
    op.drop_index(op.f("ix_match_report_competitor_id"), table_name="match_report_competitor")
    op.drop_table("match_report_competitor")
    op.drop_index(op.f("ix_match_report_id"), table_name="match_report")
    op.drop_table("match_report")

    SAEnumMatchLevel.drop(op.get_bind())
    SAEnumClassification.drop(op.get_bind())
    SAEnumDivision.drop(op.get_bind())
    SAEnumPowerFactor.drop(op.get_bind())
    SAEnumScoring.drop(op.get_bind())
    # ### end Alembic commands ###



from sqlalchemy import create_engine
from sqlalchemy.engine.url import URL
from sqlalchemy.ext.asyncio import AsyncSession, create_async_engine
from sqlalchemy.orm import sessionmaker

from . import defaults as _defaults


def get_sqlalchemy_url(
    scheme: str = _defaults.HITFACTORPY_DB_CONNECTION_SCHEME,
    username: str = _defaults.HITFACTORPY_DB_USERNAME,
    password: str = _defaults.HITFACTORPY_DB_PASSWORD,
    host: str = _defaults.HITFACTORPY_DB_HOST,
    port: int = _defaults.HITFACTORPY_DB_PORT,
    database: str = _defaults.HITFACTORPY_DB_DATABASE_NAME,
):
    return URL.create(  # type: ignore
        scheme,
        username=username,
        password=password,
        host=host,
        port=port,
        database=database,
    )


def make_async_session(sqlalchemy_url: str):
    engine = create_async_engine(sqlalchemy_url)
    return sessionmaker(bind=engine, expire_on_commit=False, class_=AsyncSession)


def make_sync_session(sqlalchemy_url: str):
    engine = create_engine(sqlalchemy_url)
    return sessionmaker(bind=engine)



HITFACTORPY_DB_SQLALCHEMY_URL = "HITFACTORPY_DB_SQLALCHEMY_URL"
HITFACTORPY_DB_ALEMBIC_DIR = "HITFACTORPY_DB_ALEMBIC_DIR"
HITFACTORPY_DB_USERNAME = "HITFACTORPY_DB_USERNAME"
HITFACTORPY_DB_PASSWORD = "HITFACTORPY_DB_PASSWORD"
HITFACTORPY_DB_HOST = "HITFACTORPY_DB_HOST"
HITFACTORPY_DB_PORT = "HITFACTORPY_DB_PORT"
HITFACTORPY_DB_DATABASE_NAME = "HITFACTORPY_DB_DATABASE_NAME"
HITFACTORPY_DB_CONNECTION_SCHEME = "HITFACTORPY_DB_CONNECTION_SCHEME"



"""add more stage_score fields

Revision ID: 9357303f57a4
Revises: bd2fc147be53
Create Date: 2023-01-26 11:43:47.040775

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "9357303f57a4"
down_revision = "bd2fc147be53"
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    from hitfactorpy_sqlalchemy.orm.models import SAEnumPowerFactor

    op.add_column("match_report_stage_score", sa.Column("raw_points", sa.Float(), nullable=True))
    op.add_column("match_report_stage_score", sa.Column("penalty_points", sa.Float(), nullable=True))
    op.add_column("match_report_stage_score", sa.Column("total_points", sa.Float(), nullable=True))
    op.add_column("match_report_stage_score", sa.Column("hit_factor", sa.Float(), nullable=True))
    op.add_column("match_report_stage_score", sa.Column("stage_points", sa.Float(), nullable=True))
    op.add_column("match_report_stage_score", sa.Column("stage_place", sa.Integer(), nullable=True))
    op.add_column(
        "match_report_stage_score",
        sa.Column(
            "stage_power_factor",
            SAEnumPowerFactor,
            nullable=True,
        ),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("match_report_stage_score", "stage_power_factor")
    op.drop_column("match_report_stage_score", "stage_place")
    op.drop_column("match_report_stage_score", "stage_points")
    op.drop_column("match_report_stage_score", "hit_factor")
    op.drop_column("match_report_stage_score", "total_points")
    op.drop_column("match_report_stage_score", "penalty_points")
    op.drop_column("match_report_stage_score", "raw_points")
    # ### end Alembic commands ###



"""require stage_number match_id uniqueness

Revision ID: bd2fc147be53
Revises: c27a53ccd1d3
Create Date: 2023-01-23 22:34:52.068605

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "bd2fc147be53"
down_revision = "c27a53ccd1d3"
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(
        "stage_score_stage_number_unique_per_match", "match_report_stage", ["match_id", "stage_number"]
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("stage_score_stage_number_unique_per_match", "match_report_stage", type_="unique")
    # ### end Alembic commands ###



import typer

from .match_report import cli as match_report_cli

cli = typer.Typer()
cli.add_typer(match_report_cli, name="match-report", help="manage match-report models")


if __name__ == "__main__":
    cli()



"""add stage number

Revision ID: c27a53ccd1d3
Revises: e3f53a281f1a
Create Date: 2023-01-23 22:08:32.065469

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "c27a53ccd1d3"
down_revision = "e3f53a281f1a"
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("match_report_stage", sa.Column("stage_number", sa.Integer()))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("match_report_stage", "stage_number")
    # ### end Alembic commands ###



import hashlib
import sys
import uuid
from dataclasses import dataclass
from decimal import Decimal
from typing import Dict

import typer
from hitfactorpy.decimal_ import D4
from hitfactorpy.parsers.match_report.pandas import parse_match_report
from rich import print
from rich.console import Console
from sqlalchemy.engine.url import URL

from .. import defaults as _defaults
from .. import env as _env

cli = typer.Typer()
stderr = Console(stderr=True)


@dataclass(frozen=True)
class CtxDbConnection:
    scheme: str
    username: str
    password: str
    host: str
    port: int
    database: str


@dataclass(frozen=True)
class CtxObj:
    db_connection: CtxDbConnection
    sqlalchemy_url: str


@cli.callback()
def inject_shared_options(
    ctx: typer.Context,
    username: str = typer.Option(
        _defaults.HITFACTORPY_DB_USERNAME, help="Username for DB connection", envvar=_env.HITFACTORPY_DB_USERNAME
    ),
    password: str = typer.Option(
        _defaults.HITFACTORPY_DB_PASSWORD, help="Password for DB connection", envvar=_env.HITFACTORPY_DB_USERNAME
    ),
    host: str = typer.Option(_defaults.HITFACTORPY_DB_HOST, help="Database host", envvar=_env.HITFACTORPY_DB_HOST),
    port: int = typer.Option(_defaults.HITFACTORPY_DB_PORT, help="Database port", envvar=_env.HITFACTORPY_DB_PORT),
    database: str = typer.Option(
        _defaults.HITFACTORPY_DB_DATABASE_NAME, help="Database name", envvar=_env.HITFACTORPY_DB_DATABASE_NAME
    ),
    scheme: str = typer.Option(
        _defaults.HITFACTORPY_DB_CONNECTION_SCHEME,
        help="SQLAlchemy database connection protocol",
        envvar=_env.HITFACTORPY_DB_CONNECTION_SCHEME,
    ),
):
    db_conn = CtxDbConnection(
        scheme=scheme,
        username=username,
        password=password,
        host=host,
        port=port,
        database=database,
    )
    ctx.obj = CtxObj(
        db_connection=db_conn,
        sqlalchemy_url=str(
            URL.create(  # type: ignore
                db_conn.scheme,
                username=db_conn.username,
                password=db_conn.password,
                host=db_conn.host,
                port=db_conn.port,
                database=db_conn.database,
            )
        ),
    )


@cli.command()
def debug(ctx: typer.Context):
    """open a debugger with an active sqlalchemy session"""
    from hitfactorpy_sqlalchemy.session import make_sync_session

    config: CtxObj = ctx.obj
    Session = make_sync_session(config.sqlalchemy_url)

    with Session() as session:
        import sqlalchemy as sa

        from hitfactorpy_sqlalchemy.orm.models import MatchReport, MatchReportStageScore

        stmt = sa.select(MatchReport)
        result = session.execute(stmt)
        match_report = result.fetchone()[0]
        print(match_report.name)

        num_stage_scores = session.query(sa.func.count(MatchReportStageScore.id)).scalar()
        print(f"total scores: {num_stage_scores}")

        stmt4 = sa.select(MatchReportStageScore).where(
            MatchReportStageScore.calculated_hit_factor == MatchReportStageScore.hit_factor
        )
        result4 = session.execute(stmt4)
        scores4 = [r[0] for r in result4.fetchall()]
        print(f"scores where hit_factor == calculated_hit_factor: {len(scores4)}")

        stmt3 = sa.select(MatchReportStageScore).where(
            MatchReportStageScore.calculated_hit_factor != MatchReportStageScore.hit_factor
        )
        result3 = session.execute(stmt3)
        scores3 = [r[0] for r in result3.fetchall()]
        print(f"scores where hit_factor != calculated_hit_factor (SQL): {len(scores3)}")
        score_mismatches_python = [s for s in scores3 if s.hit_factor != s.calculated_hit_factor]
        smp = score_mismatches_python
        print(f"scores where hit_factor != calculated_hit_factor (PYTHON): {len(smp)}")

        breakpoint()

        session.close()


@cli.command()
def match_report(
    ctx: typer.Context,
    file: typer.FileText = typer.Argument(sys.stdin, help="Report file in raw (.txt) format"),
):
    """import a match report"""
    from hitfactorpy_sqlalchemy.orm.models import (
        MatchReport,
        MatchReportCompetitor,
        MatchReportStage,
        MatchReportStageScore,
    )
    from hitfactorpy_sqlalchemy.session import make_sync_session

    config: CtxObj = ctx.obj

    file_text = "\n".join(file.readlines())
    file_hash = hashlib.md5(file_text.encode()).hexdigest()

    Session = make_sync_session(config.sqlalchemy_url)

    with Session() as session:
        # Check if match has already been imported
        existing = session.query(MatchReport).where(MatchReport.report_hash == uuid.UUID(file_hash)).count()
        if existing:
            stderr.print("report has already been imported")
            session.rollback()
            session.close()
            raise typer.Exit(1)

        parsed_match_report = parse_match_report(file_text)

        match_report = MatchReport(  # type: ignore
            name=parsed_match_report.name,
            date=parsed_match_report.date,
            match_level=parsed_match_report.match_level,
            report_hash=file_hash,
        )
        session.add(match_report)

        competitors: Dict[int, MatchReportCompetitor] = {}
        if parsed_match_report.competitors:
            for c in parsed_match_report.competitors:
                competitor_model = MatchReportCompetitor(  # type: ignore
                    member_number=c.member_number,
                    first_name=c.first_name,
                    last_name=c.last_name,
                    division=c.division,
                    power_factor=c.power_factor,
                    classification=c.classification,
                    dq=c.dq,
                    reentry=c.reentry,
                    match=match_report,
                )
                session.add(competitor_model)
                competitors[c.internal_id] = competitor_model

        stages: Dict[int, MatchReportStage] = {}
        if parsed_match_report.stages:
            for s in parsed_match_report.stages:
                stage_model = MatchReportStage(  # type: ignore
                    name=s.name,
                    min_rounds=s.min_rounds,
                    max_points=s.max_points,
                    classifier=s.classifier,
                    classifier_number=s.classifier_number,
                    scoring_type=s.scoring_type,
                    stage_number=s.internal_id,
                    match=match_report,
                )
                session.add(stage_model)
                stages[s.internal_id] = stage_model

        if parsed_match_report.stage_scores:
            for sc in parsed_match_report.stage_scores:
                stage_score_model = MatchReportStageScore(  # type: ignore
                    match=match_report,
                    competitor=competitors[sc.competitor_id or -1],
                    stage=stages[sc.stage_id or -1],
                    dq=sc.dq,
                    dnf=sc.dnf,
                    a=sc.a,
                    b=sc.b,
                    c=sc.c,
                    d=sc.d,
                    m=sc.m,
                    npm=sc.npm,
                    ns=sc.ns,
                    procedural=sc.procedural,
                    late_shot=sc.late_shot,
                    extra_shot=sc.extra_shot,
                    extra_hit=sc.extra_hit,
                    other_penalty=sc.other_penalty,
                    t1=sc.t1,
                    t2=sc.t2,
                    t3=sc.t3,
                    t4=sc.t4,
                    t5=sc.t5,
                    time=sc.time,
                    raw_points=sc.raw_points,
                    penalty_points=sc.penalty_points,
                    total_points=sc.total_points,
                    hit_factor=sc.hit_factor,
                    stage_points=sc.stage_points,
                    stage_place=sc.stage_place,
                    stage_power_factor=sc.stage_power_factor,
                )
                session.add(stage_score_model)
                if stage_score_model.calculated_hit_factor != D4(stage_score_model.hit_factor):
                    stderr.print(
                        f"found score where hf({stage_score_model.hit_factor}) != calculated_hf({stage_score_model.calculated_hit_factor})"
                    )
                    session.rollback()
                    session.close()
                    raise typer.Exit(1)

        typer.echo("Committing changes...")
        session.commit()
        session.close()


if __name__ == "__main__":
    cli()



from os import path
from pathlib import Path

HITFACTORPY_MODULE_PATH = Path(path.dirname(path.abspath(__file__)))
HITFACTORPY_DB_ALEMBIC_DIR = HITFACTORPY_MODULE_PATH / "migrations"
HITFACTORPY_DB_USERNAME = "postgres"
HITFACTORPY_DB_PASSWORD = "postgres"
HITFACTORPY_DB_HOST = "localhost"
HITFACTORPY_DB_PORT = 5432
HITFACTORPY_DB_DATABASE_NAME = "hitfactorpy"
HITFACTORPY_DB_CONNECTION_SCHEME = "postgresql+psycopg2"



"""add match report hash column

Revision ID: 20916ea34f25
Revises: c2b4247b094f
Create Date: 2023-01-23 15:41:22.251987

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "20916ea34f25"
down_revision = "c2b4247b094f"
branch_labels = None
depends_on = None

CONSTRAINT_NAME_MATCH_REPORT_COLUMN_REPORT_HASH_UNIQUE = (
    "hitfactorpy_constraint_match_report_report_hash_unique_20230123"
)


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "match_report",
        sa.Column(
            "report_hash",
            postgresql.UUID(as_uuid=True),
            nullable=True,
            comment="This represents an md5 hexdigest, stored as a UUID. Used to identify potential duplicate report imports.",
        ),
    )
    op.alter_column(
        "match_report",
        "id",
        existing_type=sa.INTEGER(),
        comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        existing_nullable=False,
        autoincrement=True,
    )
    op.alter_column(
        "match_report",
        "uuid",
        existing_type=postgresql.UUID(),
        comment="ID exposed in public APIs",
        existing_nullable=False,
        existing_server_default=sa.text("gen_random_uuid()"),
    )
    op.create_unique_constraint(CONSTRAINT_NAME_MATCH_REPORT_COLUMN_REPORT_HASH_UNIQUE, "match_report", ["report_hash"])
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(CONSTRAINT_NAME_MATCH_REPORT_COLUMN_REPORT_HASH_UNIQUE, "match_report", type_="unique")
    op.alter_column(
        "match_report",
        "uuid",
        existing_type=postgresql.UUID(),
        comment=None,
        existing_comment="ID exposed in public APIs",
        existing_nullable=False,
        existing_server_default=sa.text("gen_random_uuid()"),
    )
    op.alter_column(
        "match_report",
        "id",
        existing_type=sa.INTEGER(),
        comment=None,
        existing_comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        existing_nullable=False,
        autoincrement=True,
    )
    op.drop_column("match_report", "report_hash")
    # ### end Alembic commands ###



"""update comments

Revision ID: e3f53a281f1a
Revises: 20916ea34f25
Create Date: 2023-01-23 20:31:50.456873

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "e3f53a281f1a"
down_revision = "20916ea34f25"
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "match_report_competitor",
        "id",
        existing_type=sa.INTEGER(),
        comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        existing_nullable=False,
        autoincrement=True,
    )
    op.alter_column(
        "match_report_competitor",
        "uuid",
        existing_type=postgresql.UUID(),
        comment="ID exposed in public APIs",
        existing_nullable=False,
        existing_server_default=sa.text("gen_random_uuid()"),
    )
    op.alter_column(
        "match_report_stage",
        "id",
        existing_type=sa.INTEGER(),
        comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        existing_nullable=False,
        autoincrement=True,
        existing_server_default=sa.text("nextval('match_report_stage_id_seq'::regclass)"),
    )
    op.alter_column(
        "match_report_stage",
        "uuid",
        existing_type=postgresql.UUID(),
        comment="ID exposed in public APIs",
        existing_nullable=False,
        existing_server_default=sa.text("gen_random_uuid()"),
    )
    op.alter_column(
        "match_report_stage_score",
        "id",
        existing_type=sa.INTEGER(),
        comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        existing_nullable=False,
        autoincrement=True,
    )
    op.alter_column(
        "match_report_stage_score",
        "uuid",
        existing_type=postgresql.UUID(),
        comment="ID exposed in public APIs",
        existing_nullable=False,
        existing_server_default=sa.text("gen_random_uuid()"),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "match_report_stage_score",
        "uuid",
        existing_type=postgresql.UUID(),
        comment=None,
        existing_comment="ID exposed in public APIs",
        existing_nullable=False,
        existing_server_default=sa.text("gen_random_uuid()"),
    )
    op.alter_column(
        "match_report_stage_score",
        "id",
        existing_type=sa.INTEGER(),
        comment=None,
        existing_comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        existing_nullable=False,
        autoincrement=True,
    )
    op.alter_column(
        "match_report_stage",
        "uuid",
        existing_type=postgresql.UUID(),
        comment=None,
        existing_comment="ID exposed in public APIs",
        existing_nullable=False,
        existing_server_default=sa.text("gen_random_uuid()"),
    )
    op.alter_column(
        "match_report_stage",
        "id",
        existing_type=sa.INTEGER(),
        comment=None,
        existing_comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        existing_nullable=False,
        autoincrement=True,
        existing_server_default=sa.text("nextval('match_report_stage_id_seq'::regclass)"),
    )
    op.alter_column(
        "match_report_competitor",
        "uuid",
        existing_type=postgresql.UUID(),
        comment=None,
        existing_comment="ID exposed in public APIs",
        existing_nullable=False,
        existing_server_default=sa.text("gen_random_uuid()"),
    )
    op.alter_column(
        "match_report_competitor",
        "id",
        existing_type=sa.INTEGER(),
        comment=None,
        existing_comment="Internal ID to be used as PK and FKs. Do not expose in public APIs",
        existing_nullable=False,
        autoincrement=True,
    )
    # ### end Alembic commands ###



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hitfactorpy_sqlalchemy',
 'hitfactorpy_sqlalchemy.cli',
 'hitfactorpy_sqlalchemy.cli.model',
 'hitfactorpy_sqlalchemy.migrations',
 'hitfactorpy_sqlalchemy.migrations.versions',
 'hitfactorpy_sqlalchemy.migrations.versions copy',
 'hitfactorpy_sqlalchemy.orm']

package_data = \
{'': ['*']}

install_requires = \
['alembic>=1.9.2,<2.0.0',
 'asyncpg>=0.27.0,<0.28.0',
 'hitfactorpy>=0.1.1,<0.2.0',
 'inflection>=0.5.1,<0.6.0',
 'psycopg2-binary>=2.9.5,<3.0.0',
 'rich<13',
 'sqlalchemy-continuum>=1.3.14,<2.0.0',
 'sqlalchemy-utils>=0.39.0,<0.40.0',
 'sqlalchemy[asyncio,mypy]>=1.4,<2.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['hitfactorpy-sqlalchemy = hitfactorpy_sqlalchemy.cli:cli']}

setup_kwargs = {
    'name': 'hitfactorpy-sqlalchemy',
    'version': '0.0.5',
    'description': 'Manage practical match reports with SQLAlchemy',
    'long_description': '# hitfactorpy_sqlalchemy\n\n[![Main](https://github.com/cahna/hitfactorpy-sqlalchemy/actions/workflows/main.yaml/badge.svg)](https://github.com/cahna/hitfactorpy-sqlalchemy/actions/workflows/main.yaml)\n[![PyPI version](https://badge.fury.io/py/hitfactorpy-sqlalchemy.svg)](https://badge.fury.io/py/hitfactorpy-sqlalchemy)\n\nManage practical match reports in a database with SQLAlchemy\n\n## Status\n\n**Work in progress...**\n\n## Documentation\n\nSee `docs/index.md` via [website](https://cahna.github.io/hitfactorpy-sqlalchemy/) or [source](https://github.com/cahna/hitfactorpy-sqlalchemy/blob/main/docs/index.md).\n',
    'author': 'Conor Heine',
    'author_email': 'conor.heine@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<3.12',
}


setup(**setup_kwargs)
